package mygame;

import com.jm3.scene.Terrain;
import com.jme3.app.SimpleApplication;
import com.jme3.light.AmbientLight;
import com.jme3.material.Material;
import com.jme3.material.RenderState;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.Mesh;
import com.jme3.scene.Spatial;
import com.jme3.texture.Texture;
import com.jme3.util.SkyFactory;
import java.util.ArrayList;

/**
 * Every JME3 game is an instance of the com.jme3.app.SimpleApplication class.
 * The SimpleApplication class is the simplest example of an application: It
 * manages a 3D scene graph, checks for user input, updates the game state, and
 * automatically draws the scene to the screen. These are the core features of a
 * game engine. You extend this simple application and customize it to create
 * your game.
 *
 * @author G. Amador \& A. Gomes
 */
public class Main extends SimpleApplication {

    private final int lod = 5;
    private final int roughness = 100;
    private final float size = 500;

    /*
     * You start every JME3 game from the main() method, as every standard Java
     * application: 
     * 1 - Instantiate your SimpleApplication-based class. 
     * 2 - The application's start() method to start the game engine.
     *
     * @param args
     */
    public static void main(String[] args) {
        Main app = new Main();          // instantiate the game
        app.setDisplayFps(false);       // TODO: UNCOMMENT TO HIDE FPS INFO ON APPLICATION RUN
        app.setDisplayStatView(false);  // TODO: UNCOMMENT TO HIDE STATS VIEW INFO ON APPLICATION RUN
        //app.setShowSettings(false);     // TODO: UNCOMMENT TO HIDE SETTINGS MENU ON STARTUP
        app.start();                    // start the game
    }

    /**
     * Method that is automatically called once at the beginning when the
     * application starts. Every JME3 game must have this method. In the
     * simpleInitApp() method, you load game objects before the game starts.
     */
    @Override
    public void simpleInitApp() {
        // We speedup the movement of the flyby camera default camera
        flyCam.setMoveSpeed(100);

        /* create a SkyBox of dimension */
        /**
         * BEGIN OF EXERCISE 1 (Step 1): uncomment code as indicated in comments
         * in the following TODO's for Step 1.
         */
        // TODO: (Sub-Step 1.1) UNCOMMENT THE FOLLOWING BLOCK
/*        
        Texture westTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_west.jpg");
        Texture eastTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_east.jpg");
        Texture northTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_north.jpg");
        Texture southTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_south.jpg");
        Texture upTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_up.jpg");
        Texture downTex = assetManager.loadTexture("Textures/Skybox/SkyBox2_down.jpg");

        final Vector3f normalScale = new Vector3f(-1, 1, 1);
        Spatial skySpatial = SkyFactory.createSky(
                assetManager,
                westTex,
                eastTex,
                northTex,
                southTex,
                upTex,
                downTex,
                normalScale);
        rootNode.attachChild(skySpatial);
*/
        // END OF EXERCISE 1 (Step 1). -----------------------------------------        

        /*Create terrain material and load four textures into it.*/
        /**
         * BEGIN OF EXERCISE 2 (Step 2): complete the diamond square algorithm
         * in the following TODO's for Step 2.
         */
        Terrain t = new Terrain(lod, roughness, size);
        t.generateTerrain();
        ArrayList<Mesh> meshesTerrain = t.getTerrain();

        /* Using the Mesh in a Scene */
        /**
         * We create a com.jme3.scene.Geometry and com.jme3.material.Material
         * from our mesh, apply a simple color material to it or perform vertex
         * coloring, and attach it to the rootNode to make it appear in the
         * scene.
         */
        //Using the mesh together with the Unshaded.j3md material.
        Material mat = new Material(assetManager, "Common/MatDefs/Misc/Unshaded.j3md");
        //activate wireframe
        mat.getAdditionalRenderState().setWireframe(true);
        // set the same color for all vertexes
        mat.setBoolean("VertexColor", true);
        for (Mesh m : meshesTerrain) {
            Geometry geoTerrain = new Geometry("Terrain", m); // using our custom mesh object  
            geoTerrain.setMaterial(mat);
            geoTerrain.setLocalTranslation(-size / 2, -size / 10, -size / 2);
            rootNode.attachChild(geoTerrain);
        }


        /* Debugging Tip: Culling */
        /**
         * By default, jME3 optimizes a mesh by “back face culling”, this means
         * not drawing the inside. It determines the side of a triangle by the
         * order of the vertices: The front face is the face where the vertices
         * are specified counter-clockwise. This means for you that, by default,
         * your custom mesh is invisible when seen from “behind” or from the
         * inside. This may not be a problem, typically this is even intended,
         * because it's faster. The player will not look at the inside of most
         * things anyway. For example, if your custom mesh is a closed
         * polyhedron, or a flat wallpaper-like object, then rendering the back
         * faces (the inside of the pillar, the back of the painting, etc) would
         * indeed be a waste of resources. In case however that your use case
         * requires the back faces be visible, you have two options: If you have
         * a very simple scene, you can simply deactivate back face culling for
         * this one mesh's material.
         */
        mat.getAdditionalRenderState().setFaceCullMode(RenderState.FaceCullMode.Off);

        // END OF EXERCISE 2 (Step 2). -----------------------------------------  

        /**
         * add ambient light. Since, ambient light does not in reality exist we
         * shall call it god because like ambient light god is everywhere ...
         * assuming god exists obviously. If it/he/she does or doesn't exist is
         * in the scope of theology not science.
         *
         */
        AmbientLight god = new AmbientLight(); //because god is light and god is everywere 
        //got..setDirection(new Vector3f(-10.0f, -20.0f, -10.0f));    // The direction where the light is pointing (-1, -1, -1) and the light position (10, 20, 10)
        rootNode.addLight(god);
    }
}
